sdkp = LibStub("AceAddon-3.0"):NewAddon("sdkp", "AceConsole-3.0", "AceEvent-3.0", "AceComm-3.0", "AceSerializer-3.0", "AceTimer-3.0", "AceHook-3.0");

local optionsFrame
-- Some globals used in sdkpdata
sdkp.COLOR_UNKNOWN_CLASS = { ["r"]=0.5, ["g"]=0.5, ["b"]=0.5, ["a"]=1.0,};
sdkp.COLOR_LOWPRIORITY = { ["r"]=0.75, ["g"]=0.0, ["b"]=0.0, ["a"]=1.0,};
sdkp.COLOR_MEDIUMPRIORITY = { ["r"]=1.0, ["g"]=1.0, ["b"]=0.0, ["a"]=1.0,};
sdkp.COLOR_HIGHPRIORITY = { ["r"]=0.0, ["g"]=1.0, ["b"]=0.0, ["a"]=1.0,};


sdkp.FUNC_COLOR_BRACKET = function(namecol, maincol, tdacol, ...)
	local tdatt = tdacol.value;
	if tdatt >= 80.0 then
		return sdkp.COLOR_HIGHPRIORITY;    -- >= 70% 30 day
	elseif tdatt >= 60 then 
		return sdkp.COLOR_MEDIUMPRIORITY;  --  < 70% 30 day
	else
		return sdkp.COLOR_LOWPRIORITY;
	end
end

sdkp.FUNC_DKP_TOTAL = function(cols, earned, adjustments, spent)
	local dkp = earned + adjustments;
	
	if sdkp.db.profile.loot then 
		for _, lootdata in pairs(sdkp.db.profile.loot) do 
			-- if looter is player or player's main
			if lootdata["player"]:lower() == cols[1].value:lower() or lootdata["player"]:lower() == cols[2].value:lower() then 
				sdkp:Debug("Found purchase by", lootdata["player"], "for", lootdata["price"], tonumber(lootdata["price"]) or 0);
				spent = spent + (tonumber(lootdata["price"]) or 0); -- dkp - price of loot won.
			end
		end
	end
	
	if sdkp:IsRelationalDkp() then 
		spent = spent + sdkp:GetInitialGearValue();
		if spent == 0 then 
			spent = 1;
		end
		dkp = dkp / spent;
	else
		dkp = dkp - spent;
	end
	return tonumber(string.format("%.3f", dkp));
end

sdkp.FUNC_PLAYER_BID = function(namecol, ...)
	-- local bid = sdkp.db.profile.bids ? sdkp.db.profile.bids[namecol.value:lower()] : "";
	local bid = sdkp.db.profile.bids and sdkp.db.profile.bids[namecol.value:lower()] or "";
	return bid or "";
end

function sdkp:OnInitialize()
	self.db = LibStub("AceDB-3.0"):New("SDKPDB", SDKP_DEFAULTS, "Default")
    LibStub("AceConfig-3.0"):RegisterOptionsTable("sdkp", SDKP_OPTIONS);
    self.optionsFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("sdkp", "sdkp");
    self:RegisterChatCommand("sdkp", "ChatCommand");
    if SDKP_DATA_INIT then
    	SDKP_DATA_INIT();
    	if not sdkp.db.profile.currentsystem then 
    		for system in pairs(SDKPDATA) do
    			sdkp.db.profile.currentsystem = system;
    			break;
    		end
   		end
    else
    	self:Print("Could not locate dkp data.  Please download the sdkpdata.lua file and place it in the Interface\\AddOns\\sdkp folder.");
    end
    --OrigChatFrameOnEvent = ChatFrame_OnEvent;
    --ChatFrame_OnEvent = HookChatFrameOnEvent;
    self.BIDDING = false;
    self:SecureHook("SetItemRef");
    self:RawHook("ChatFrame_OnEvent", true); -- whisper commands
    self.bb = LibStub("LibBabble-Boss-3.0"):GetReverseLookupTable();
    SDKP_RAID_ZONE["Scarlet Monastery"] = self.db.profile.debug; -- easy to test on sm bosses, close to zone.
    
    for i, snapshot in ipairs(self.db.profile.snapshots) do 
    	self:AddRaidFilter(snapshot.name, snapshot.playertable);
    end
    
    self:ShowBids();
    self.newui:Hide();

	-- initialize ui values
	if self.db and self.db.profile.currentfilter then 
    	self.newui.filtercontrol:Fire("OnValueChanged", self.db.profile.currentfilter);
    end
    
    -- initialize variables 
    self.penaltybox = {};
end

function sdkp:OnEnable()
    self:Print("Enabled.");
    self:RegisterEvent("CHAT_MSG_LOOT");
	self:RegisterEvent("PLAYER_REGEN_ENABLED");
	self:RegisterEvent("CHAT_MSG_WHISPER");
	self:RegisterEvent("LOOT_OPENED");
	self:RegisterEvent("ZONE_CHANGED_NEW_AREA");
	self:RegisterComm(self.name, "OnCommReceive");
	
	self:RegisterMessage("SDKP_LOOT_ASSIGNED");
end
function sdkp:OnDisable()
    self:Print("Disabled.");
end

function sdkp:GetDkpForItem(item)
	local ilvl, dkp = 0, 0;
	if ilvlDKP then 
		ilvl, dkp = ilvlDKP(item);
	else
		self:Print("You should install ItemLevelDkp...")
	end
	return dkp;
end

function sdkp:SendWhisper(who, what)
	SendChatMessage(self.name .. ": " .. what, "WHISPER", nil, who);	
end
function sdkp:SendOfficer(what)
	SendChatMessage(self.name .. ": " .. what, "OFFICER");	
end
function sdkp:SendRaid(what)
	SendChatMessage(self.name .. ": " .. what, "RAID");	
end
function sdkp:SendRaidWarning(what)
	SendChatMessage(self.name .. ": " .. what, "RAID_WARNING");	
end
function sdkp:SendGuild(what)
	SendChatMessage(self.name .. ": " .. what, "GUILD");	
end

function sdkp:GetAltInfo(player)
	local playerindex = SDKPPLAYER[player:lower()];
	if not playerindex then
		local i, numGuildMembers = 1, GetNumGuildMembers(true);
		for i = 1, numGuildMembers do 
			local name, _, _, _, _, _, _, officernote, _, _ = GetGuildRosterInfo(i);
			if name:lower() == player:lower() then
				local _, _, main = string.find(officernote, "^Main:%s+([^%s]+)");
				if main then 
					sdkp:Debug("GetAltInfo:", name, officernote, main);
					sdkp:Debug("GetAltInfo (altmatch):", player, "=>", main);
					return main;
				end
			end
		end
	else
		local pdata = SDKPDATA[sdkp.db.profile.currentsystem][playerindex];
		if pdata.cols[2].value ~= "" then
			sdkp:Debug("GetAltInfo (doigethere?):", player, "=>", pdata.cols[2].value);
			return pdata.cols[2].value;
		elseif pdata.cols[1].value:lower() == player:lower() then
			sdkp:Debug("GetAltInfo (mainmatch):", player, "=>", pdata.cols[1].value);
			return pdata.cols[1].value;			
		end
	end
	sdkp:Debug("GetAltInfo (failed):", player, "=>", player);
	return player;
end

function sdkp:getPlayerData(player)
	local playerindex = SDKPPLAYER[player:lower()];
	local pdata = nil;
	if playerindex then 
		pdata = SDKPDATA[sdkp.db.profile.currentsystem][playerindex];
	else
		local main = self:GetAltInfo(player);
		playerindex = SDKPPLAYER[main:lower()];
		if playerindex then 
			pdata = SDKPDATA[sdkp.db.profile.currentsystem][SDKPPLAYER[main:lower()]];
		end
	end
	return pdata;
end

function sdkp:getPlayerRank(player)
	local i, numGuildMembers = 1, GetNumGuildMembers(true);
	for i = 1, numGuildMembers do 
		local name, rankString, rankIndex, _, _, _, _, _, _, _ = GetGuildRosterInfo(i);
		if name:lower() == player:lower() then
			sdkp:Debug("GetRankInfo:", name, rankString, rankIndex);
			return rankString, rankIndex;
		end
	end
end

function sdkp:Debug(...)
	if self:IsDebug() then 
		self:Print("Debug: ", ...)
	end
end

-- Hooks
function sdkp:SetItemRef(link, text, button)
	local biditem = getglobal("sdkpeditbox");
	if biditem and not sdkp.BIDDING and link:find("^item:") then
		biditem:SetText(text);
	end
end

function sdkp:ChatFrame_OnEvent(context, event, msg, ...)
	-- this hook is used to hide unwanted messages from sight.
	if event == "CHAT_MSG_WHISPER_INFORM" and msg:find("^" .. sdkp.name) then
		-- responding to a whisper command.
		sdkp:Debug("event", event, "msg", msg, ...);
	elseif event == "CHAT_MSG_WHISPER" and 
		(msg:find("^!help") or msg:find("^!here") or msg:find("^!dkp") 
		or msg:find("^!attendance") or msg:find("^!att")
		or msg:find("^!waitlist") or msg:find("^!wl")) then
		-- hide incoming whisper commands
		sdkp:Debug("event", event, "msg", msg, ...);
	else
		-- call original handler
		self.hooks["ChatFrame_OnEvent"](context, event, msg, ...);	
	end
end

function sdkp:RefreshScrollTables()
	if self.BIDSCROLLFRAME then
    	self.BIDSCROLLFRAME:SortData();
	end
	if self.LOOTDETAILFRAME then
    	self.LOOTDETAILFRAME:SortData();
	end	
	
	if self.newui ~= nil then 
		self.newui.filtercontrol:SetList(SDKP_FILTERS_LIST);
	end
end

function sdkp:AddRaidFilter(sRaidNote, tPlayerTable)
	local sLabel = "   " .. sRaidNote;
	if sLabel ~= SDKP_FILTERS_LIST[#SDKP_FILTERS_LIST] then 
		table.insert(SDKP_FILTERS_LIST, sLabel);
		SDKP_FILTERS[sLabel] = function(self, row)
			return tPlayerTable[row.cols[1].value];	
		end;
		self:RefreshScrollTables();
	end
end